const express = require('express');
const fs = require('fs');
const path = require('path');

const app = express();
const PORT = 3000;

// Directory to index
const baseDirectory = path.join(__dirname, 'files'); // Change 'files' to your target folder

app.get('/', (req, res) => {
    const dir = req.query.dir ? path.join(baseDirectory, req.query.dir) : baseDirectory;

    // Validate directory path
    if (!dir.startsWith(baseDirectory) || !fs.existsSync(dir) || !fs.statSync(dir).isDirectory()) {
        return res.status(400).send('Invalid directory.');
    }

    // Get directory contents
    const items = fs.readdirSync(dir);

    // Generate HTML
    let html = `
        <!DOCTYPE html>
        <html lang="en">
        <head>
            <meta charset="UTF-8">
            <meta name="viewport" content="width=device-width, initial-scale=1.0">
            <title>Directory Autoindex</title>
            <style>
                body { font-family: Arial, sans-serif; margin: 20px; }
                h1 { text-align: center; color: #333; }
                table { width: 100%; border-collapse: collapse; margin-top: 20px; }
                th, td { border: 1px solid #ccc; padding: 10px; text-align: left; }
                th { background-color: #f4f4f4; }
                tbody tr:nth-child(odd) td { background-color: #e6f7ff; }
                tbody tr:nth-child(even) td { background-color: #ffffff; }
                a { text-decoration: none; color: blue; }
                a:hover { text-decoration: underline; }
                .no-data { text-align: center; color: #666; }
            </style>
        </head>
        <body>
            <h1>Directory Autoindex</h1>
            <table>
                <thead>
                    <tr>
                        <th>Sl. No</th>
                        <th>File/Directory Name</th>
                        <th>Size</th>
                        <th>View</th>
                        <th>Download</th>
                    </tr>
                </thead>
                <tbody>
    `;

    if (items.length > 0) {
        items.forEach((item, index) => {
            const itemPath = path.join(dir, item);
            const isFile = fs.statSync(itemPath).isFile();
            const size = isFile ? `${(fs.statSync(itemPath).size / 1024).toFixed(2)} KB` : 'Folder';

            html += `
                <tr>
                    <td>${index + 1}</td>
                    <td>
                        ${
                            isFile
                                ? item
                                : `<a href="?dir=${encodeURIComponent(path.relative(baseDirectory, itemPath))}">${item}/</a>`
                        }
                    </td>
                    <td>${size}</td>
                    <td>${isFile ? `<a href="/view?file=${encodeURIComponent(path.relative(baseDirectory, itemPath))}" target="_blank">View</a>` : '-'}</td>
                    <td>${isFile ? `<a href="/download?file=${encodeURIComponent(path.relative(baseDirectory, itemPath))}">Download</a>` : '-'}</td>
                </tr>
            `;
        });
    } else {
        html += `
            <tr>
                <td colspan="5" class="no-data">No files or directories found.</td>
            </tr>
        `;
    }

    html += `
                </tbody>
            </table>
        </body>
        </html>
    `;

    res.send(html);
});

app.get('/view', (req, res) => {
    const file = path.join(baseDirectory, req.query.file);
    if (file.startsWith(baseDirectory) && fs.existsSync(file) && fs.statSync(file).isFile()) {
        res.sendFile(file);
    } else {
        res.status(400).send('Invalid file.');
    }
});

app.get('/download', (req, res) => {
    const file = path.join(baseDirectory, req.query.file);
    if (file.startsWith(baseDirectory) && fs.existsSync(file) && fs.statSync(file).isFile()) {
        res.download(file);
    } else {
        res.status(400).send('Invalid file.');
    }
});

app.listen(PORT, () => {
    console.log(`Server running at http://localhost:${PORT}`);
});
